'use strict';

var path = require('path');
var camelcase = require('lodash.camelcase');
var rollupBabel = require('rollup-plugin-babel');
var commonjs = require('rollup-plugin-commonjs');
var nodeResolve = require('rollup-plugin-node-resolve');
var json = require('rollup-plugin-json');
var uglify = require('rollup-plugin-uglify');
var rollupAlias = require('rollup-plugin-alias');
var nodeBuiltIns = require('rollup-plugin-node-builtins');
var nodeGlobals = require('rollup-plugin-node-globals');

var _require = require('../utils'),
    pkg = _require.pkg,
    hasFile = _require.hasFile,
    hasPkgProp = _require.hasPkgProp,
    parseEnv = _require.parseEnv;

var here = function here(p) {
  return path.join(__dirname, p);
};
var capitalize = function capitalize(s) {
  return s[0].toUpperCase() + s.slice(1);
};

var minify = parseEnv('BUILD_MINIFY', false);
var format = process.env.BUILD_FORMAT;
var isPreact = parseEnv('BUILD_PREACT', false);
var isNode = parseEnv('BUILD_NODE', false);
var name = process.env.BUILD_NAME || capitalize(camelcase(pkg.name));

var defaultGlobals = Object.keys(pkg.peerDependencies || {}).reduce(function (deps, dep) {
  deps[dep] = capitalize(camelcase(dep));
  return deps;
}, {});

var defaultExternal = Object.keys(pkg.peerDependencies || {});

var filenameSuffix = parseEnv('BUILD_FILENAME_SUFFIX', isPreact ? '.preact' : '');
var globals = parseEnv('BUILD_GLOBALS', isPreact ? Object.assign(defaultGlobals, { preact: 'preact' }) : defaultGlobals);
var external = parseEnv('BUILD_EXTERNAL', isPreact ? defaultExternal.concat(['preact', 'prop-types']) : defaultExternal).filter(function (e, i, arry) {
  return arry.indexOf(e) === i;
});

if (isPreact) {
  delete globals.react;
  delete globals['prop-types']; // TODO: is this necessary?
  external.splice(external.indexOf('react'), 1);
}

var alias = parseEnv('BUILD_ALIAS', isPreact ? { react: 'preact' } : null);
var esm = format === 'esm';
var umd = format === 'umd';
var cjs = format === 'cjs';

var output = void 0;

var filename = path.join('dist', pkg.name);

if (esm) {
  output = [{ file: `${filename}${filenameSuffix}.es.js`, format: 'es' }];
} else if (umd) {
  if (minify) {
    output = [{
      file: `${filename}${filenameSuffix}.umd.min.js`,
      format: 'umd',
      sourcemap: true
    }];
  } else {
    output = [{
      file: `${filename}${filenameSuffix}.umd.js`,
      format: 'umd',
      sourcemap: true
    }];
  }
} else if (cjs) {
  output = [{ file: `${filename}${filenameSuffix}.cjs.js`, format: 'cjs' }];
} else if (format) {
  throw new Error(`invalid format specified: "${format}".`);
} else {
  throw new Error('no format specified. --environment FORMAT:xxx');
}

var useBuiltinConfig = !hasFile('.babelrc') && !hasPkgProp('babel');
var babelPresets = useBuiltinConfig ? [here('../config/babelrc.js')] : [];

module.exports = {
  input: 'src/index.js',
  output,
  exports: esm ? 'named' : 'default',
  name,
  external,
  globals,
  plugins: [alias ? rollupAlias(alias) : null, isNode ? nodeBuiltIns() : null, isNode ? nodeGlobals() : null, nodeResolve({ preferBuiltins: isNode, jsnext: true, main: true }), commonjs({ include: 'node_modules/**' }), json(), rollupBabel({
    exclude: 'node_modules/**',
    presets: babelPresets,
    babelrc: true
  }), minify ? uglify() : null].filter(Boolean)
};